# This file was auto-generated by Fern from our API Definition.

import typing

TransactionLedgerType = typing.Union[
    typing.Literal[
        "ASSET",
        "LIABILITY",
        "EQUITY",
        "REVENUE",
        "EXPENSE",
        "INTEREST_INCOME",
        "COGS_EXPENSE",
        "SGA_EXPENSE",
        "SM_EXPENSE",
        "RD_EXPENSE",
        "INTEREST_EXPENSE",
        "TAX_EXPENSE",
        "COMMODITY",
    ],
    typing.Any,
]
