# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...types.transaction import Transaction
import typing_extensions
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class GetTransactionsByMonthResponse(UniversalBaseModel):
    transactions: typing.Optional[typing.List[Transaction]] = None
    total_count: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="totalCount")] = pydantic.Field(
        default=None
    )
    """
    Total number of transactions
    """

    month: typing.Optional[str] = pydantic.Field(default=None)
    """
    Month in YYYY-MM format
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
