# This file was auto-generated by Fern from our API Definition.

import typing

TransactionCategorizationStatus = typing.Union[typing.Literal["PENDING", "READY_FOR_INPUT", "CATEGORIZED"], typing.Any]
