# This file was auto-generated by Fern from our API Definition.

import typing

TransactionTransactionType = typing.Union[
    typing.Literal["INVOICE", "PAYMENT", "BANK_TRANSACTION", "BANK_TRANSACTION_PLAID"], typing.Any
]
