# This file was auto-generated by Fern from our API Definition.

import typing

CompanyEntityType = typing.Union[
    typing.Literal["LLC", "CORPORATION", "PARTNERSHIP", "SOLE_PROPRIETORSHIP", "NON_PROFIT"], typing.Any
]
